// arb_d2xx.cpp : Defines the entry point for the console application.
//

#include <stdio.h>
#include <windows.h>
#include "ftd2xx.h"


short data[16000];
char outdata[64000];

const char LF   = 0x0A;
const char CR   = 0x0D;
const char ESC  = 0x1B;
const char PLUS = 0x2B;


/*---------------------------------------------------------------------------*/
bool
IsSpecial(const char c)
{
    // CR, LF, ESC and PLUS characters have special meaning and
    // must be escaped if present in data
    return ((c == CR)||(c == LF)||(c == ESC)||(c == PLUS));
}


/*---------------------------------------------------------------------------*/
bool Write(FT_HANDLE handle, const char* buffer, size_t len)
{
    unsigned long bytesWritten;

    FT_STATUS status = FT_Write( handle, (LPVOID)buffer, (unsigned long)len, &bytesWritten );

    if ( ( status != FT_OK ) || ( bytesWritten != len ) )
        {
        printf( "Unable to write to controller. Status: %d\r\n", status );
        return false;
        }

    return true;
}


/*---------------------------------------------------------------------------*/
void
CheckError(FT_HANDLE handle)
{
    char* cmd = "SYST:ERR?\n";
    Write(handle, cmd, strlen( cmd ));

    cmd = "++read eoi\n";
    Write(handle, cmd, strlen( cmd ));

    char resp[100];
    unsigned long bytesRead;
    FT_Read( handle, &resp, sizeof( resp ), &bytesRead );
    resp[bytesRead] = 0;

    if (bytesRead)
    {
        printf("%s", resp);
    }
}


/*---------------------------------------------------------------------------*/
int main(int argc, char* argv[])
{
    FT_HANDLE handle;

    if (argc != 3)
    {
        printf("Usage: arb_d2xx <HP33120A GPIB address> <Number of data points>\n");
        return 0;
    }

    int gpibaddr = atoi(argv[1]);
    int numpoints = atoi(argv[2]);

    if (numpoints < 8)
    {
        printf("Too few points\n");
        return 0;
    }

    if (numpoints > 16000)
    {
        printf("Too many points\n");
        return 0;
    }

    // Build arbitrary waveform data
    int k = 0;
    while (k < (numpoints/2))
    {
        data[k++] = (short)((k * 2047)/(numpoints/2));
    }

    int j = k;
    while (j > 0)
    {
        data[k++] = data[--j];
    }

    // Convert 16-bit waveform data points to big-endian byte order
    // and escape CR, LF, ESC and '+' characters
    int i = 0;
    for(k = 0 ; k < numpoints; k++)
    {
        char c = (char)((data[k] >> 8) & 0xFF);
        if (IsSpecial(c))
        {
            outdata[i++] = ESC;
        }
        outdata[i++] = c;

        c = char(data[k] & 0xFF);
        if (IsSpecial(c))
        {
            outdata[i++] = ESC;
        }
        outdata[i++] = c;
    }


    printf( "Connecting to Prologix GPIB-USB Controller...\r\n" );
    FT_STATUS status = FT_OpenEx( "Prologix GPIB-USB Controller", FT_OPEN_BY_DESCRIPTION, &handle );
    if ( status != FT_OK )
        {
        printf( "Unable to connect to controller.\r\n" );
        return 0;
        }

    printf( "Successfully connected.\r\n" );

    // 500 msec read timeout. No write timeout.
    status = FT_SetTimeouts( handle, 500, 0 );

    if ( status != FT_OK )
        {
        printf( "Unable to set timeouts. Status: %d\r\n", status );
        FT_Close( handle );
        return 0;
        }

    // Controller mode
    char* cmd = "++mode 1\n";
    Write(handle, cmd, strlen( cmd ));

    // GPIB address of HP 33120A
    char buffer[32];
    sprintf( buffer, "++addr %d\n", gpibaddr);
    Write(handle, buffer, strlen( buffer ));

    // No automatic read-after-write
    cmd = "++auto 0\n";
    Write(handle, cmd, strlen( cmd ));

    // No GPIB termination characters appended
    cmd = "++eos 3\n";
    Write(handle, cmd, strlen( cmd ));

    // Assert EOI with last byte
    cmd = "++eoi 1\n";
    Write(handle, cmd, strlen( cmd ));

    // Following commands load waveform data.
    // See HP 33120A manual for details
    cmd = "*RST\n";
    printf( "%s", cmd);
    Write(handle, cmd, strlen( cmd ));
    Sleep(1000);
    CheckError(handle);

    char datalen[10];
    sprintf(datalen, "%d", numpoints*2);
    sprintf( buffer, "DATA:DAC VOLATILE, #%d%s", strlen(datalen), datalen);
    printf("%s\n", buffer);

    Write(handle, buffer, strlen( buffer ));
    Write(handle, outdata, i);
    Write(handle, "\n", 1);

    Sleep(500);
    CheckError(handle);

    cmd = "DATA:COPY PULSE, VOLATILE\n";
    printf( "%s", cmd);
    Write(handle, cmd, strlen( cmd ));
    Sleep(2000);
    CheckError(handle);

    cmd = "FUNC:USER PULSE\n";
    printf( "%s", cmd);
    Write(handle, cmd, strlen( cmd ));
    Sleep(500);
    CheckError(handle);

    cmd = "FUNC:SHAP USER\n";
    printf( "%s", cmd);
    Write(handle, cmd, strlen( cmd ));
    Sleep(500);
    CheckError(handle);

    cmd = "OUTP:LOAD 50\n";
    printf( "%s", cmd);
    Write(handle, cmd, strlen( cmd ));
    Sleep(500);
    CheckError(handle);

    cmd = "FREQ 5000;VOLT 0.5\n";
    printf( "%s", cmd);
    Write(handle, cmd, strlen( cmd ));
    Sleep(500);
    CheckError(handle);

    FT_Close( handle );

    return 0;
}

